@@ -1,7 +1,7 @@
 /* Copyright 2003 Roger Dingledine
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
+ * Copyright 2004-2006 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: torint.h,v 1.19 2005/04/01 20:15:54 nickm Exp $ */
+/* $Id: torint.h,v 1.24 2006/03/29 05:14:12 weasel Exp $ */
 
 /**
  * \file torint.h
@@ -10,7 +10,7 @@
 
 #ifndef __TORINT_H
 #define __TORINT_H
-#define TORINT_H_ID "$Id: torint.h,v 1.19 2005/04/01 20:15:54 nickm Exp $"
+#define TORINT_H_ID "$Id: torint.h,v 1.24 2006/03/29 05:14:12 weasel Exp $"
 
 #include "orconfig.h"
 
@@ -27,10 +27,13 @@
 #include <sys/limits.h>
 #endif
 #ifdef HAVE_MACHINE_LIMITS_H
-#ifndef __FreeBSD__
+#if !defined(__FreeBSD__) && !defined(__FreeBSD_kernel__)
   /* FreeBSD has a bug where it complains that this file is obsolete,
      and I should migrate to using sys/limits. It complains even when
-     I include both. */
+     I include both.
+     __FreeBSD_kernel__ is defined by Debian GNU/kFreeBSD which
+      does the same thing (but doesn't defined __FreeBSD__).
+     */
 #include <machine/limits.h>
 #endif
 #endif
@@ -224,6 +227,16 @@ typedef uint32_t uintptr_t;
 #endif
 #endif
 
+#ifndef INT_MAX
+#if (SIZEOF_INT == 4)
+#define INT_MAX 0x7fffffffL
+#elif (SIZEOF_INT == 8)
+#define INT_MAX 0x7fffffffffffffffL
+#else
+#error "Can't define INT_MAX"
+#endif
+#endif
+
 #ifndef UINT_MAX
 #if (SIZEOF_INT == 2)
 #define UINT_MAX 0xffffu
@@ -237,14 +250,28 @@ typedef uint32_t uintptr_t;
 #endif
 
 #ifndef TIME_MAX
+
+#ifdef TIME_T_IS_SIGNED
+
+#if (SIZEOF_TIME_T == SIZEOF_INT)
+#define TIME_MAX ((time_t)INT_MAX)
+#elif (SIZEOF_TIME_T == SIZEOF_LONG)
+#define TIME_MAX ((time_t)LONG_MAX)
+#else
+#error "Can't define (signed) TIME_MAX"
+#endif
+
+#else
+/* Unsigned case */
 #if (SIZEOF_TIME_T == 4)
 #define TIME_MAX ((time_t)UINT32_MAX)
 #elif (SIZEOF_TIME_T == 8)
 #define TIME_MAX ((time_t)UINT64_MAX)
 #else
-#error "Can't define TIME_MAX"
-#endif
+#error "Can't define (unsigned) TIME_MAX"
 #endif
+#endif /* time_t_is_signed */
+#endif /* ifndef(TIME_MAX) */
 
 /* Any size_t larger than this amount is likely to be an underflow. */
 #define SIZE_T_CEILING (sizeof(char)<<(sizeof(size_t)*8 - 1))
