@@ -2,9 +2,9 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_or.c,v 1.214 2006/03/05 09:50:25 nickm Exp $ */
+/* $Id: connection_or.c,v 1.216 2006/03/18 01:24:04 arma Exp $ */
 const char connection_or_c_id[] =
-  "$Id: connection_or.c,v 1.214 2006/03/05 09:50:25 nickm Exp $";
+  "$Id: connection_or.c,v 1.216 2006/03/18 01:24:04 arma Exp $";
 
 /**
  * \file connection_or.c
@@ -16,6 +16,7 @@ const char connection_or_c_id[] =
 
 /** How much clock skew do we tolerate when checking certificates for
  * known routers? (sec) */
+
 #define TIGHT_CERT_ALLOW_SKEW (90*60)
 
 static int connection_tls_finish_handshake(connection_t *conn);
@@ -461,8 +462,8 @@ connection_or_connect(uint32_t addr, uint16_t port, const char *id_digest)
        * an https proxy, our https proxy is down. Don't blame the
        * Tor server. */
       if (!options->HttpsProxy) {
-        router_mark_as_down(conn->identity_digest);
         entry_guard_set_status(conn->identity_digest, 0);
+        router_set_status(conn->identity_digest, 0);
       }
       control_event_or_conn_status(conn, OR_CONN_EVENT_FAILED);
       connection_free(conn);
@@ -636,6 +637,7 @@ connection_or_check_valid_handshake(connection_t *conn, char *digest_rcvd)
              "but got %s",
              conn->address, conn->port, expected, seen);
       entry_guard_set_status(conn->identity_digest, 0);
+      router_set_status(conn->identity_digest, 0);
       control_event_or_conn_status(conn, OR_CONN_EVENT_FAILED);
       as_advertised = 0;
     }
@@ -702,6 +704,7 @@ connection_tls_finish_handshake(connection_t *conn)
       /* pending circs get closed in circuit_about_to_close_connection() */
       return -1;
     }
+    router_set_status(conn->identity_digest, 1);
   }
   connection_watch_events(conn, EV_READ);
   circuit_n_conn_done(conn, 1); /* send the pending creates, if any. */
