@@ -1,9 +1,9 @@
 /* Copyright 2003-2004 Roger Dingledine.
  * Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dns.c,v 1.182 2006/03/19 03:55:48 arma Exp $ */
+/* $Id: dns.c,v 1.182.2.1 2006/04/18 03:05:34 nickm Exp $ */
 const char dns_c_id[] =
-  "$Id: dns.c,v 1.182 2006/03/19 03:55:48 arma Exp $";
+  "$Id: dns.c,v 1.182.2.1 2006/04/18 03:05:34 nickm Exp $";
 
 /**
  * \file dns.c
@@ -725,7 +725,7 @@ connection_dns_process_inbuf(connection_t *conn)
       connection_fetch_from_buf(&success,1,conn);
       connection_fetch_from_buf((char *)&addr,sizeof(uint32_t),conn);
       log_warn(LD_EXIT,"Discarding idle dns answer (success %d, addr %d.)",
-               success, addr); // XXX safe_str
+               success, addr);
     }
     return 0;
   }
