@@ -2,9 +2,9 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerparse.c,v 1.179 2006/03/27 02:25:34 arma Exp $ */
+/* $Id: routerparse.c,v 1.183 2006/04/10 07:50:38 nickm Exp $ */
 const char routerparse_c_id[] =
-  "$Id: routerparse.c,v 1.179 2006/03/27 02:25:34 arma Exp $";
+  "$Id: routerparse.c,v 1.183 2006/04/10 07:50:38 nickm Exp $";
 
 /**
  * \file routerparse.c
@@ -1926,8 +1926,24 @@ _compare_tor_version_str_ptr(const void **_a, const void **_b)
 
 /** Sort a list of string-representations of versions in ascending order. */
 void
-sort_version_list(smartlist_t *versions)
+sort_version_list(smartlist_t *versions, int remove_duplicates)
 {
+  int i;
+
   smartlist_sort(versions, _compare_tor_version_str_ptr);
+  if (!remove_duplicates)
+    return;
+
+  for (i = 1; i < smartlist_len(versions); ++i) {
+    const void *a, *b;
+    a = smartlist_get(versions, i-1);
+    b = smartlist_get(versions, i);
+    /* use version_cmp so we catch multiple representations of the same
+     * version */
+    if (_compare_tor_version_str_ptr(&a,&b) == 0) {
+      tor_free(smartlist_get(versions, i));
+      smartlist_del_keeporder(versions, i--);
+    }
+  }
 }
 
